/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#include "celtpriv.h"



/*CELT_LOG2_FRAC_TABLE[k]=celt_log2_frac(k+1,3)
  There are at most 23 bands in any of the custom modes, with a size at most
   28 when LM=0.*/
const unsigned char CELT_LOG2_FRAC_TABLE[28]={
   0,
   8,13,
  16,19,21,23,
  24,26,27,28,29,30,31,32,
  32,33,34,34,35,36,36,37,37,38,38,39,39
};

/*floor(16384*2^(k/(1<<CELT_BITRES)))*/
const opus_int16 CELT_EXP2_TABLE_Q14[1<<CELT_BITRES]={
  16384,17866,19483,21247,23170,25267,27554,30048
};


/*For each V(N,K) supported, we will access element U(min(N,K+1),max(N,K+1)).
  Thus, the number of entries in row I is the larger of the maximum number of
   pulses we will ever allocate for a given N=I (K=128, or however many fit in
   32 bits, whichever is smaller), plus one, and the maximum N for which
   K=I-1 pulses fit in 32 bits (limited to the set of N which can be achieved
   by splitting a band from an Opus mode: 176, 144, 96, 88, 72, 64, 48, 44, 36,
   32, 24, 22, 18, 16, 8, 4, 2).*/
const opus_uint32 CELT_PVQ_U_DATA[1272]={
  /*N=0, K=0...176:*/
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  /*N=1, K=1...176:*/
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  /*N=2, K=2...176:*/
  3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41,
  43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 71, 73, 75, 77, 79,
  81, 83, 85, 87, 89, 91, 93, 95, 97, 99, 101, 103, 105, 107, 109, 111, 113,
  115, 117, 119, 121, 123, 125, 127, 129, 131, 133, 135, 137, 139, 141, 143,
  145, 147, 149, 151, 153, 155, 157, 159, 161, 163, 165, 167, 169, 171, 173,
  175, 177, 179, 181, 183, 185, 187, 189, 191, 193, 195, 197, 199, 201, 203,
  205, 207, 209, 211, 213, 215, 217, 219, 221, 223, 225, 227, 229, 231, 233,
  235, 237, 239, 241, 243, 245, 247, 249, 251, 253, 255, 257, 259, 261, 263,
  265, 267, 269, 271, 273, 275, 277, 279, 281, 283, 285, 287, 289, 291, 293,
  295, 297, 299, 301, 303, 305, 307, 309, 311, 313, 315, 317, 319, 321, 323,
  325, 327, 329, 331, 333, 335, 337, 339, 341, 343, 345, 347, 349, 351,
  /*N=3, K=3...176:*/
  13, 25, 41, 61, 85, 113, 145, 181, 221, 265, 313, 365, 421, 481, 545, 613,
  685, 761, 841, 925, 1013, 1105, 1201, 1301, 1405, 1513, 1625, 1741, 1861,
  1985, 2113, 2245, 2381, 2521, 2665, 2813, 2965, 3121, 3281, 3445, 3613, 3785,
  3961, 4141, 4325, 4513, 4705, 4901, 5101, 5305, 5513, 5725, 5941, 6161, 6385,
  6613, 6845, 7081, 7321, 7565, 7813, 8065, 8321, 8581, 8845, 9113, 9385, 9661,
  9941, 10225, 10513, 10805, 11101, 11401, 11705, 12013, 12325, 12641, 12961,
  13285, 13613, 13945, 14281, 14621, 14965, 15313, 15665, 16021, 16381, 16745,
  17113, 17485, 17861, 18241, 18625, 19013, 19405, 19801, 20201, 20605, 21013,
  21425, 21841, 22261, 22685, 23113, 23545, 23981, 24421, 24865, 25313, 25765,
  26221, 26681, 27145, 27613, 28085, 28561, 29041, 29525, 30013, 30505, 31001,
  31501, 32005, 32513, 33025, 33541, 34061, 34585, 35113, 35645, 36181, 36721,
  37265, 37813, 38365, 38921, 39481, 40045, 40613, 41185, 41761, 42341, 42925,
  43513, 44105, 44701, 45301, 45905, 46513, 47125, 47741, 48361, 48985, 49613,
  50245, 50881, 51521, 52165, 52813, 53465, 54121, 54781, 55445, 56113, 56785,
  57461, 58141, 58825, 59513, 60205, 60901, 61601,
  /*N=4, K=4...176:*/
  63, 129, 231, 377, 575, 833, 1159, 1561, 2047, 2625, 3303, 4089, 4991, 6017,
  7175, 8473, 9919, 11521, 13287, 15225, 17343, 19649, 22151, 24857, 27775,
  30913, 34279, 37881, 41727, 45825, 50183, 54809, 59711, 64897, 70375, 76153,
  82239, 88641, 95367, 102425, 109823, 117569, 125671, 134137, 142975, 152193,
  161799, 171801, 182207, 193025, 204263, 215929, 228031, 240577, 253575,
  267033, 280959, 295361, 310247, 325625, 341503, 357889, 374791, 392217,
  410175, 428673, 447719, 467321, 487487, 508225, 529543, 551449, 573951,
  597057, 620775, 645113, 670079, 695681, 721927, 748825, 776383, 804609,
  833511, 863097, 893375, 924353, 956039, 988441, 1021567, 1055425, 1090023,
  1125369, 1161471, 1198337, 1235975, 1274393, 1313599, 1353601, 1394407,
  1436025, 1478463, 1521729, 1565831, 1610777, 1656575, 1703233, 1750759,
  1799161, 1848447, 1898625, 1949703, 2001689, 2054591, 2108417, 2163175,
  2218873, 2275519, 2333121, 2391687, 2451225, 2511743, 2573249, 2635751,
  2699257, 2763775, 2829313, 2895879, 2963481, 3032127, 3101825, 3172583,
  3244409, 3317311, 3391297, 3466375, 3542553, 3619839, 3698241, 3777767,
  3858425, 3940223, 4023169, 4107271, 4192537, 4278975, 4366593, 4455399,
  4545401, 4636607, 4729025, 4822663, 4917529, 5013631, 5110977, 5209575,
  5309433, 5410559, 5512961, 5616647, 5721625, 5827903, 5935489, 6044391,
  6154617, 6266175, 6379073, 6493319, 6608921, 6725887, 6844225, 6963943,
  7085049, 7207551,
  /*N=5, K=5...176:*/
  321, 681, 1289, 2241, 3649, 5641, 8361, 11969, 16641, 22569, 29961, 39041,
  50049, 63241, 78889, 97281, 118721, 143529, 172041, 204609, 241601, 283401,
  330409, 383041, 441729, 506921, 579081, 658689, 746241, 842249, 947241,
  1061761, 1186369, 1321641, 1468169, 1626561, 1797441, 1981449, 2179241,
  2391489, 2618881, 2862121, 3121929, 3399041, 3694209, 4008201, 4341801,
  4695809, 5071041, 5468329, 5888521, 6332481, 6801089, 7295241, 7815849,
  8363841, 8940161, 9545769, 10181641, 10848769, 11548161, 12280841, 13047849,
  13850241, 14689089, 15565481, 16480521, 17435329, 18431041, 19468809,
  20549801, 21675201, 22846209, 24064041, 25329929, 26645121, 28010881,
  29428489, 30899241, 32424449, 34005441, 35643561, 37340169, 39096641,
  40914369, 42794761, 44739241, 46749249, 48826241, 50971689, 53187081,
  55473921, 57833729, 60268041, 62778409, 65366401, 68033601, 70781609,
  73612041, 76526529, 79526721, 82614281, 85790889, 89058241, 92418049,
  95872041, 99421961, 103069569, 106816641, 110664969, 114616361, 118672641,
  122835649, 127107241, 131489289, 135983681, 140592321, 145317129, 150160041,
  155123009, 160208001, 165417001, 170752009, 176215041, 181808129, 187533321,
  193392681, 199388289, 205522241, 211796649, 218213641, 224775361, 231483969,
  238341641, 245350569, 252512961, 259831041, 267307049, 274943241, 282741889,
  290705281, 298835721, 307135529, 315607041, 324252609, 333074601, 342075401,
  351257409, 360623041, 370174729, 379914921, 389846081, 399970689, 410291241,
  420810249, 431530241, 442453761, 453583369, 464921641, 476471169, 488234561,
  500214441, 512413449, 524834241, 537479489, 550351881, 563454121, 576788929,
  590359041, 604167209, 618216201, 632508801,
  /*N=6, K=6...96 (technically V(109,5) fits in 32 bits, but that can't be
     achieved by splitting an Opus band):*/
  1683, 3653, 7183, 13073, 22363, 36365, 56695, 85305, 124515, 177045, 246047,
  335137, 448427, 590557, 766727, 982729, 1244979, 1560549, 1937199, 2383409,
  2908411, 3522221, 4235671, 5060441, 6009091, 7095093, 8332863, 9737793,
  11326283, 13115773, 15124775, 17372905, 19880915, 22670725, 25765455,
  29189457, 32968347, 37129037, 41699767, 46710137, 52191139, 58175189,
  64696159, 71789409, 79491819, 87841821, 96879431, 106646281, 117185651,
  128542501, 140763503, 153897073, 167993403, 183104493, 199284183, 216588185,
  235074115, 254801525, 275831935, 298228865, 322057867, 347386557, 374284647,
  402823977, 433078547, 465124549, 499040399, 534906769, 572806619, 612825229,
  655050231, 699571641, 746481891, 795875861, 847850911, 902506913, 959946283,
  1020274013, 1083597703, 1150027593, 1219676595, 1292660325, 1369097135,
  1449108145, 1532817275, 1620351277, 1711839767, 1807415257, 1907213187,
  2011371957, 2120032959,
  /*N=7, K=7...54 (technically V(60,6) fits in 32 bits, but that can't be
     achieved by splitting an Opus band):*/
  8989, 19825, 40081, 75517, 134245, 227305, 369305, 579125, 880685, 1303777,
  1884961, 2668525, 3707509, 5064793, 6814249, 9041957, 11847485, 15345233,
  19665841, 24957661, 31388293, 39146185, 48442297, 59511829, 72616013,
  88043969, 106114625, 127178701, 151620757, 179861305, 212358985, 249612805,
  292164445, 340600625, 395555537, 457713341, 527810725, 606639529, 695049433,
  793950709, 904317037, 1027188385, 1163673953, 1314955181, 1482288821,
  1667010073, 1870535785, 2094367717,
  /*N=8, K=8...37 (technically V(40,7) fits in 32 bits, but that can't be
     achieved by splitting an Opus band):*/
  48639, 108545, 224143, 433905, 795455, 1392065, 2340495, 3800305, 5984767,
  9173505, 13726991, 20103025, 28875327, 40754369, 56610575, 77500017,
  104692735, 139703809, 184327311, 240673265, 311207743, 398796225, 506750351,
  638878193, 799538175, 993696769, 1226990095, 1505789553, 1837271615,
  2229491905U,
  /*N=9, K=9...28 (technically V(29,8) fits in 32 bits, but that can't be
     achieved by splitting an Opus band):*/
  265729, 598417, 1256465, 2485825, 4673345, 8405905, 14546705, 24331777,
  39490049, 62390545, 96220561, 145198913, 214828609, 312193553, 446304145,
  628496897, 872893441, 1196924561, 1621925137, 2173806145U,
  /*N=10, K=10...24:*/
  1462563, 3317445, 7059735, 14218905, 27298155, 50250765, 89129247, 152951073,
  254831667, 413442773, 654862247, 1014889769, 1541911931, 2300409629U,
  3375210671U,
  /*N=11, K=11...19 (technically V(20,10) fits in 32 bits, but that can't be
     achieved by splitting an Opus band):*/
  8097453, 18474633, 39753273, 81270333, 158819253, 298199265, 540279585,
  948062325, 1616336765,
  /*N=12, K=12...18:*/
  45046719, 103274625, 224298231, 464387817, 921406335, 1759885185,
  3248227095U,
  /*N=13, K=13...16:*/
  251595969, 579168825, 1267854873, 2653649025U,
  /*N=14, K=14:*/
  1409933619
};

const opus_uint32 *const CELT_PVQ_U_ROW[15]={
  CELT_PVQ_U_DATA+   0,CELT_PVQ_U_DATA+ 176,CELT_PVQ_U_DATA+ 351,
  CELT_PVQ_U_DATA+ 525,CELT_PVQ_U_DATA+ 698,CELT_PVQ_U_DATA+ 870,
  CELT_PVQ_U_DATA+1041,CELT_PVQ_U_DATA+1131,CELT_PVQ_U_DATA+1178,
  CELT_PVQ_U_DATA+1207,CELT_PVQ_U_DATA+1226,CELT_PVQ_U_DATA+1240,
  CELT_PVQ_U_DATA+1248,CELT_PVQ_U_DATA+1254,CELT_PVQ_U_DATA+1257
};



/*Parameters of the Laplace-like probability models used for the coarse energy.
  There is one pair of parameters for each frame size, prediction type
   (inter/intra), and band number.
  The first number of each pair is the probability of 0, and the second is the
   decay rate, both in Q8 precision.*/
const unsigned char CELT_E_PROB_MODEL[4][2][42]={
  /*120 sample frames.*/
  {
    /*Inter*/
    {
       72, 127,  65, 129,  66, 128,  65, 128,  64, 128,  62, 128,  64, 128,
       64, 128,  92,  78,  92,  79,  92,  78,  90,  79, 116,  41, 115,  40,
      114,  40, 132,  26, 132,  26, 145,  17, 161,  12, 176,  10, 177,  11
    },
    /*Intra*/
    {
       24, 179,  48, 138,  54, 135,  54, 132,  53, 134,  56, 133,  55, 132,
       55, 132,  61, 114,  70,  96,  74,  88,  75,  88,  87,  74,  89,  66,
       91,  67, 100,  59, 108,  50, 120,  40, 122,  37,  97,  43,  78,  50
    }
  },
  /*240 sample frames.*/
  {
    /*Inter*/
    {
       83,  78,  84,  81,  88,  75,  86,  74,  87,  71,  90,  73,  93,  74,
       93,  74, 109,  40, 114,  36, 117,  34, 117,  34, 143,  17, 145,  18,
      146,  19, 162,  12, 165,  10, 178,   7, 189,   6, 190,   8, 177,   9
    },
    /*Intra*/
    {
       23, 178,  54, 115,  63, 102,  66,  98,  69,  99,  74,  89,  71,  91,
       73,  91,  78,  89,  86,  80,  92,  66,  93,  64, 102,  59, 103,  60,
      104,  60, 117,  52, 123,  44, 138,  35, 133,  31,  97,  38,  77,  45
    }
  },
  /*480 sample frames.*/
  {
    /*Inter*/
    {
       61,  90,  93,  60, 105,  42, 107,  41, 110,  45, 116,  38, 113,  38,
      112,  38, 124,  26, 132,  27, 136,  19, 140,  20, 155,  14, 159,  16,
      158,  18, 170,  13, 177,  10, 187,   8, 192,   6, 175,   9, 159,  10
    },
    /*Intra*/
    {
       21, 178,  59, 110,  71,  86,  75,  85,  84,  83,  91,  66,  88,  73,
       87,  72,  92,  75,  98,  72, 105,  58, 107,  54, 115,  52, 114,  55,
      112,  56, 129,  51, 132,  40, 150,  33, 140,  29,  98,  35,  77,  42
    }
  },
  /*960 sample frames.*/
  {
    /*Inter*/
    {
       42, 121,  96,  66, 108,  43, 111,  40, 117,  44, 123,  32, 120,  36,
      119,  33, 127,  33, 134,  34, 139,  21, 147,  23, 152,  20, 158,  25,
      154,  26, 166,  21, 173,  16, 184,  13, 184,  10, 150,  13, 139,  15
    },
    /*Intra*/
    {
       22, 178,  63, 114,  74,  82,  84,  83,  92,  82, 103,  62,  96,  72,
       96,  67, 101,  73, 107,  72, 113,  55, 118,  52, 125,  52, 118,  52,
      117,  55, 135,  49, 137,  39, 157,  32, 145,  29,  97,  33,  77,  40
    }
  }
};


const float CELT_E_ALPHA[4]={
  29440/32768.0F,26112/32768.0F,21248/32768.0F,16384/32768.0F
};

const float CELT_E_BETA[4]={
  30147/32768.0F,22282/32768.0F,12124/32768.0F, 6554/32768.0F
};

const float CELT_E_MEANS[25]={
  6.4375F,6.25F,  5.75F, 5.3125F,5.0625F,
  4.8125F,4.5F,   4.375F,4.875F, 4.6875F,
  4.5625F,4.4375F,4.875F,4.625F, 4.3125F,
  4.5F,   4.375F, 4.625F,4.75F,  4.4375F,
  3.75F,  3.75F,  3.75F, 3.75F,  3.75F
};



/*TF resolution change for [LM][is_transient][tf_select][band_flag].
  Positive values increase frequency resolution (and decrease time resolution);
   negative values increase time resolution (and decrease frequency
   resolution).
  Only transients can increase time resolution.
  The LM==0 frame size can never increase time resolution, because it can't be
   a transient.*/
const signed char CELT_TF_SELECT[4][2][2][2]={
  { { { 0,-1},{ 0,-1} },{ { 0,-1},{ 0,-1} } },
  { { { 0,-1},{ 0,-2} },{ { 1, 0},{ 1,-1} } },
  { { { 0,-2},{ 0,-3} },{ { 2, 0},{ 1,-1} } },
  { { { 0,-2},{ 0,-3} },{ { 3, 0},{ 1,-1} } }
};



const unsigned char CELT_SMALL_ENERGY_PDF[3]={2,1,1};


const unsigned char CELT_SPREAD_PDF[4]={7,2,21,2};


const unsigned char CELT_TRIM_PDF[11]={2,2,5,10,22,46,22,10,5,2,2};


const unsigned char CELT_BIT_INTERLEAVE_TABLE[16]={
  0,1,1,1,2,3,3,3,2,3,3,3,2,3,3,3
};

const unsigned char CELT_BIT_DEINTERLEAVE_TABLE[16]={
  0x00,0x03,0x0C,0x0F,0x30,0x33,0x3C,0x3F,
  0xC0,0xC3,0xCC,0xCF,0xF0,0xF3,0xFC,0xFF
};

const unsigned char CELT_HADAMARD_ORDER_TABLE[30]={
   1, 0,
   3, 0, 2, 1,
   7, 0, 4, 3, 6, 1, 5, 2,
  15, 0, 8, 7,12, 3,11, 4,14, 1, 9, 6,13, 2,10, 5
};


const float CELT_RSQRT_M[4]={
  1.0F,0.70710678F,0.5F,0.35355339F
};

const float CELT_TAPSETS[3][3]={
  {10048/32768.0F,7112/32768.0F,4248/32768.0F},
  {15200/32768.0F,8784/32768.0F,0.0F},
  {26208/32768.0F,3280/32768.0F,0.0F}
};
