/*Copyright (C) 2003-2010, Mark Borgerding.
  Copyright (C) 2010,      Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#if !defined(KISS_FFT_H)
# define KISS_FFT_H (1)
# include <stdlib.h>
# include <math.h>
# include "opus.h"



# if !defined(kiss_fft_scalar)
/*Default is float.*/
#  define kiss_fft_scalar float
# endif



typedef struct kiss_fft_cpx kiss_fft_cpx;
typedef struct kiss_fft_cfg kiss_fft_cfg;



struct kiss_fft_cpx{
  kiss_fft_scalar r;
  kiss_fft_scalar i;
};



/*E.g. an fft of length 128 has 4 factors as far as kissfft is concerned:
   4*4*4*2.*/
# define KISS_MAXFACTORS (8)

struct kiss_fft_cfg{
  /*The number of points in the FFT.*/
  int                 n_fft;
  /*The initial stride used to access the twiddle array.
    This allows the same twiddles to be shared across multiple configurations.*/
  int                 tw_stride;
  /*The scale factor to apply during the inverse FFT.*/
  kiss_fft_scalar     scale;
  /*The partition of n_fft into factor pairs (p_i,m_i), such that
     m_{i-1}=p_i*m_i, m_{-1}=n_fft, and p_i is 2, 3, 4, or 5.*/
  opus_int16          factors[2*KISS_MAXFACTORS];
  /*The table used to bitreverse the input.
    Unlike a normal bitreverse table, this is designed to operate in place.
    At each step, element i of the input is swapped with bitrev[i], after which
     element i is in the right place.
    Later elements of bitrev[] take this swapping into account to locate the
     proper value (in particular bitrev[i]>=i for all i).*/
  const opus_int16   *bitrev;
  /*twiddles[k*tw_stride]=cos(2*pi*(k/n_fft))+i*sin(2*pi*(k/n_fft))*/
  const kiss_fft_cpx *twiddles;
};

/** Perform an in-place FFT on a complex input buffer.
 * For a forward FFT,
 *  _f should be  f[0], f[1], ..., f[nfft-1]  on input.
 *  _f will be    F[0], F[1], ..., F[nfft-1]  on output.
 * Note that each element is complex and can be accessed like f[k].r and f[k].i
 */
void kiss_fft(const kiss_fft_cfg *_st,kiss_fft_cpx *_f);

#endif
