/*Copyright (C) 2003-2010, Mark Borgerding.
  Copyright (C) 2010,      Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#if !defined(KISS_FFT_GUTS_H)
# define KISS_FFT_GUTS_H (1)
# include "celtpriv.h"
/*kiss_fft.h
  defines kiss_fft_scalar as either short or a float type and defines
  typedef struct{
    kiss_fft_scalar r;
    kiss_fft_scalar i;
  }kiss_fft_cpx;*/
# include "kiss_fft.h"



/*Explanation of macros dealing with complex math:

    C_SET(_res,_r,_i)         : _res=_r+i*_i
  Cannot share _res with any other arguments:
    C_MUL(_res,_a,_b)         : _res=_a*_b;
    C_MULC(_res,_a,_b)        : _res=_a*~_b;
    C_MUL_S(_res,_s,_c)       : _res=_s*_c;
    C_MUL_SI(_res,_s,_c)      : _res=i*_s*_c;
  May share _res with _a, but not _b:
    C_MLA_S(_res,_a,_s,_b)    : _res=_a+_s*_b;
    C_MLA_SI(_res,_a,_s,_b)   : _res=_a+i*_s*_b;
  May share _res with _a or _b:
    C_ADD(_res,_a,_b)         : _res=_a+_b;
    C_SUB(_res,_a,_b)         : _res=_a-_b;
    C_SUBHALF(_res,_a,_b)     : _res=_a-0.5*_b;
  May share _res with _a, but not _b:
    C_ADDI(_res,_a,_b)        : _res=_a+i*_b;
    C_SUBI(_res,_a,_b)        : _res=_a-i*_b;*/

# if !defined(CHECK_OVERFLOW_OP)
#  define CHECK_OVERFLOW_OP(_a,_op,_b) /*NOOP*/
# endif

/*Set.*/
# define C_SET(_res,_r,_i) \
  do { \
    (_res).r=(_r); \
    (_res).i=(_i); \
  } \
  while(0)
/*Multiply.*/
# define C_MUL(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r*(_b).r,-,(_a).i*(_b).i) \
    CHECK_OVERFLOW_OP((_a).i*(_b).r,+,(_a).r*(_b).i) \
    (_res).r=(_a).r*(_b).r-(_a).i*(_b).i; \
    (_res).i=(_a).i*(_b).r+(_a).r*(_b).i; \
  } \
  while(0)
/*Multiply by the complement.*/
# define C_MULC(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r*(_b).r,+,(_a).i*(_b).i) \
    CHECK_OVERFLOW_OP((_a).i*(_b).r,-,(_a).r*(_b).i) \
    (_res).r=(_a).r*(_b).r+(_a).i*(_b).i; \
    (_res).i=(_a).i*(_b).r-(_a).r*(_b).i; \
  } \
  while(0)
/*Multiply by a real scalar.*/
# define C_MUL_S(_res,_s,_c) \
  do{ \
    (_res).r=(_s)*(_c).r; \
    (_res).i=(_s)*(_c).i; \
  } \
  while(0)
/*Multiply by an imaginary scalar.*/
# define C_MUL_SI(_res,_s,_c) \
  do{ \
    (_res).r=-(_s)*(_c).i; \
    (_res).i=(_s)*(_c).r; \
  } \
  while(0)
/*Multiply by a real scalar and accumulate.*/
# define C_MLA_S(_res,_a,_s,_b) \
  do{ \
    (_res).r=(_a).r+(_s)*(_b).r; \
    (_res).i=(_a).i+(_s)*(_b).i; \
  } \
  while(0)
/*Multiply by an imaginary scalar and accumulate.*/
# define C_MLA_SI(_res,_a,_s,_b) \
  do{ \
    (_res).r=(_a).r-(_s)*(_b).i; \
    (_res).i=(_a).i+(_s)*(_b).r; \
  } \
  while(0)
/*Add.*/
# define C_ADD(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r,+,(_b).r) \
    CHECK_OVERFLOW_OP((_a).i,+,(_b).i) \
    (_res).r=(_a).r+(_b).r; \
    (_res).i=(_a).i+(_b).i; \
  } \
  while(0)
/*Subtract.*/
# define C_SUB(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r,-,(_b).r) \
    CHECK_OVERFLOW_OP((_a).i,-,(_b).i) \
    (_res).r=(_a).r-(_b).r; \
    (_res).i=(_a).i-(_b).i; \
  } \
  while(0)
/*Subtract after scaling the second value by 1/2.*/
# define C_SUBHALF(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r,-,0.5F*(_b).r) \
    CHECK_OVERFLOW_OP((_a).i,-,0.5F*(_b).i) \
    (_res).r=(_a).r-0.5F*(_b).r; \
    (_res).i=(_a).i-0.5F*(_b).i; \
  } \
  while(0)
/*Add after scaling the second value by i.*/
# define C_ADDI(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r,-,(_b).i) \
    CHECK_OVERFLOW_OP((_a).i,+,(_b).r) \
    (_res).r=(_a).r-(_b).i; \
    (_res).i=(_a).i+(_b).r; \
  } \
  while(0)
/*Subtract after scaling the second value by i.*/
# define C_SUBI(_res,_a,_b) \
  do{ \
    CHECK_OVERFLOW_OP((_a).r,+,(_b).i) \
    CHECK_OVERFLOW_OP((_a).i,-,(_b).r) \
    (_res).r=(_a).r+(_b).i; \
    (_res).i=(_a).i-(_b).r; \
  } \
  while(0)

#endif
