/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#if !defined(OPUSPRIV_H)
# define OPUSPRIV_H (1)
#include <stddef.h>
#include <limits.h>
#include "silk.h"
#include "celt.h"



# define OPUS_MIN(_a,_b)      ((_a)<(_b)?(_a):(_b))
# define OPUS_MAX(_a,_b)      ((_a)>(_b)?(_a):(_b))
/*Clamps _b to lie between _a and _c.
  When _a>_c, this returns _a; this behavior is required.*/
# define OPUS_CLAMP(_a,_b,_c) (OPUS_MAX(_a,OPUS_MIN(_b,_c)))
# define OPUS_ABS(_x)         ((_x)<0?-(_x):(_x))
# define OPUS_SIGN(_x)        (((_x)>0)-((_x)<0))



typedef opus_uint32             opus_bit_window;

typedef struct opus_range_coder opus_range_coder;



/*OPT: This needs to be at least 32 bits, but if you have a larger type with
   fast arithmetic, it will make the decoder go faster to use it.*/
#define OPUS_BIT_WINDOW_SIZE ((int)sizeof(opus_bit_window)*CHAR_BIT)



struct opus_range_coder{
  const unsigned char *buf;
  opus_uint32          storage;
  opus_uint32          end_offs;
  opus_bit_window      end_window;
  int                  nend_bits;
  int                  nbits_total;
  opus_uint32          offs;
  opus_uint32          rng;
  opus_uint32          dif;
  opus_uint32          ext;
  int                  rem;
  int                  error;
};


void orc_dec_init(opus_range_coder *_rc,
 const unsigned char *_buf,opus_uint32 _storage);

unsigned orc_decode(opus_range_coder *_rc,unsigned _ft);
void orc_dec_update(opus_range_coder *_rc,
 unsigned _fl,unsigned _fh,unsigned _ft);
opus_uint32 orc_dec_raw_bits(opus_range_coder *_rc,int _nbits);

int orc_dec_bit_logp(opus_range_coder *_rc,unsigned _logp);
int orc_dec_pdf(opus_range_coder *_rc,const unsigned char *_pdf,unsigned _ftb);
opus_uint32 orc_dec_uint(opus_range_coder *_rc,opus_uint32 _ft);

opus_uint32 orc_tell(opus_range_coder *_rc);
opus_uint32 orc_tell_frac(opus_range_coder *_rc);



struct opus_stream_decoder{
  silk_decoder   *silkdec;
  celt_decoder   *celtdec;
  opus_read_func  read;
  void           *stream;
  opus_uint32     packet_length;
  opus_uint32     padding_length;
  opus_int16      frame_lengths[48];
  opus_uint32     rng;
  int             nframes;
  int             toc;
};

#endif
