/*Copyright (C) 2010-2011, Mozilla Corporation.
  Copyright (C) 2001-2011, Timothy B. Terriberry.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#include "opuspriv.h"



/*OPT: Most CPUs have a single instruction that does this.*/
static int opus_ilog(opus_uint32 _v){
  int ret;
  for(ret=0;_v;ret++)_v>>=1;
  return ret;
}


static int orc_read_byte(opus_range_coder *_rc){
  return _rc->offs<_rc->storage?_rc->buf[_rc->offs++]:0;
}

static int orc_read_end_byte(opus_range_coder *_rc){
  return _rc->end_offs<_rc->storage?_rc->buf[_rc->storage-++(_rc->end_offs)]:0;
}

static void orc_dec_normalize(opus_range_coder *_rc){
  while(_rc->rng<=0x800000){
    int code_bits;
    _rc->nbits_total+=8;
    _rc->rng<<=8;
    code_bits=_rc->rem;
    _rc->rem=orc_read_byte(_rc);
    code_bits=code_bits<<8|_rc->rem;
    _rc->dif=(_rc->dif<<8)+(0xFF&~(code_bits>>1))&0x7FFFFFFF;
  }
}


void orc_dec_init(opus_range_coder *_rc,
 const unsigned char *_buf,opus_uint32 _storage){
  _rc->buf=_buf;
  _rc->storage=_storage;
  _rc->end_offs=0;
  _rc->end_window=0;
  _rc->nend_bits=0;
  _rc->nbits_total=9;
  _rc->offs=0;
  _rc->rng=0x80;
  _rc->rem=orc_read_byte(_rc);
  _rc->dif=0x7F-(_rc->rem>>1);
  _rc->error=0;
  orc_dec_normalize(_rc);
}


unsigned orc_decode(opus_range_coder *_rc,unsigned _ft){
  unsigned s;
  _rc->ext=_rc->rng/_ft;
  s=(unsigned)(_rc->dif/_rc->ext);
  return _ft-OPUS_MIN(s+1,_ft);
}

void orc_dec_update(opus_range_coder *_rc,
 unsigned _fl,unsigned _fh,unsigned _ft){
  opus_uint32 s;
  s=_rc->ext*(_ft-_fh);
  _rc->dif-=s;
  _rc->rng=_fl>0?_rc->ext*(_fh-_fl):_rc->rng-s;
  orc_dec_normalize(_rc);
}

opus_uint32 orc_dec_raw_bits(opus_range_coder *_rc,int _nbits){
  opus_uint32 ret;
  if(_rc->nend_bits<_nbits){
    do{
      _rc->end_window|=
       (opus_bit_window)orc_read_end_byte(_rc)<<_rc->nend_bits;
      _rc->nend_bits+=8;
    }
    while(_rc->nend_bits<=OPUS_BIT_WINDOW_SIZE-8);
  }
  ret=(opus_uint32)_rc->end_window&((opus_uint32)1<<_nbits)-1;
  _rc->end_window>>=_nbits;
  _rc->nend_bits-=_nbits;
  _rc->nbits_total+=_nbits;
  return ret;
}

/*OPT: This can be done without any multiplies or divides, but is done here
   with the generic routines to demonstrate their compatibility.*/
int orc_dec_bit_logp(opus_range_coder *_rc,unsigned _logp){
  unsigned ft;
  int      ret;
  ft=1<<_logp;
  ret=orc_decode(_rc,ft)>=ft-1;
  orc_dec_update(_rc,ret?ft-1:0,ret?ft:ft-1,ft);
  return ret;
}

/*OPT: This can be done without any divides, but is done here with the generic
   routines to demonstrate their compatibility.*/
int orc_dec_pdf(opus_range_coder *_rc,const unsigned char *_pdf,unsigned _ftb){
  unsigned fs;
  unsigned cdf;
  int      ret;
  fs=orc_decode(_rc,1<<_ftb);
  cdf=_pdf[0];
  ret=0;
  while(fs>=cdf)cdf+=_pdf[++ret];
  orc_dec_update(_rc,cdf-_pdf[ret],cdf,1<<_ftb);
  return ret;
}

opus_uint32 orc_dec_uint(opus_range_coder *_rc,opus_uint32 _ft){
  unsigned fs;
  int      ftb;
  ftb=opus_ilog(--_ft);
  if(ftb>8){
    unsigned    ft;
    opus_uint32 ret;
    ftb-=8;
    ft=(unsigned)(_ft>>ftb)+1;
    fs=orc_decode(_rc,ft);
    orc_dec_update(_rc,fs,fs+1,ft);
    ret=(opus_uint32)fs<<ftb|orc_dec_raw_bits(_rc,ftb);
    if(ret<=_ft)return ret;
    _rc->error=1;
    return _ft;
  }
  fs=orc_decode(_rc,(unsigned)++_ft);
  orc_dec_update(_rc,fs,fs+1,(unsigned)_ft);
  return fs;
}

opus_uint32 orc_tell(opus_range_coder *_rc){
  return _rc->nbits_total-opus_ilog(_rc->rng);
}

opus_uint32 orc_tell_frac(opus_range_coder *_rc){
  opus_uint32 r;
  int         l;
  int         n;
  l=opus_ilog(_rc->rng);
  r=_rc->rng>>l-16;
  for(n=3;n-->0;){
    int b;
    r=r*r>>15;
    b=(int)(r>>16);
    l=l<<1|b;
    r>>=b;
  }
  return (_rc->nbits_total<<3)-l;
}
